package fi.jyu.intellij.comtest.rider

import com.intellij.openapi.project.Project
import comtest.v2.csharp.ComTestNUnitBackend
import comtest.v2.processors.*
import comtest.v2.utils.Strings
import fi.jyu.intellij.comtest.common.util.Notifier
import fi.jyu.intellij.comtest.rider.util.riderSolutionFile

/**
 * Wrapper object for ComTest (v2).
 *
 * For now, only supports generating NUnit tests.
 */
internal object ComTestRunner {
    init {
        val backend = ComTestNUnitBackend()
        FileProcessor.register(Language.Csharp, CsFileProcessor(backend, ComTestLoggerWindow.ComTestLogger))
        FileProcessor.register(Language.CsharpCore, CsFileProcessor(backend, ComTestLoggerWindow.ComTestLogger))
        FileProcessor.register(Language.CsharpProject, CsProjectProcessor(backend, ComTestLoggerWindow.ComTestLogger))
        FileProcessor.register(Language.CsharpSolution, CsSolutionProcessor(backend, ComTestLoggerWindow.ComTestLogger))
    }

    /**
     * Generate tests for .NET solution.
     *
     * @param proj Solution for which to generate the tests.
     * @param alwaysShowConsole If true, the ComTest console is always shown.
     * @param csprojNames List of csproj names to generate tests for. If null, generate tests for all projects.
     */
    fun generateForDotNet(
        proj: Project,
        alwaysShowConsole: Boolean = false,
        csprojNames: List<String>? = null
    ): ComTestResult? {
        ComTestLoggerWindow.open(proj, alwaysShowConsole)

        val slnFile = proj.riderSolutionFile
            ?: run {
                Notifier.notifyError(
                    proj,
                    "No solution found or solution is virtual (virtual solutions are not supported). Load a solution to generate tests."
                )
                return null
            }

        val strings = csprojNames?.takeIf { it.isNotEmpty() }?.let { Strings(it) }
        val slnEntry = FileEntry(slnFile.path, Language.CsharpSolution, strings)
        return FileProcessor.process(slnEntry, slnFile.parent, false)
    }

}